/**

  Spago4Q - The Business Intelligence Free Platform

  Copyright (C) 2009 Engineering Ingegneria Informatica S.p.A.

  Spago4Q is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Spago4Q is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago4q.extractors.msprojectclient;

import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFUserError;
import it.eng.spago4q.extractors.AbstractExtractor;
import it.eng.spago4q.extractors.bo.GenericItem;
import it.eng.spago4q.extractors.bo.GenericItemInterface;
import it.eng.spago4q.extractors.msprojectclient.bo.Project;
import it.eng.spago4q.extractors.msprojectclient.bo.Task;

import java.io.File;
import java.io.FileInputStream;
import java.util.ArrayList;
import java.util.List;

import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

public class MSProjectClientExtractor extends AbstractExtractor {

	private List<GenericItemInterface> result = new ArrayList<GenericItemInterface>();
	private static final String FILE = "FILE";
	private static final String TASK_ = "TASK_";
	private static final String PROJECT_ = "PROJECT_";
	private String document;

	protected void generateResult() throws EMFUserError {
		try {
			File file = new File(getDocument());
			InputSource src = new InputSource(new FileInputStream(file));
			SaxMSProjectClientUnmarshaller saxUms = new SaxMSProjectClientUnmarshaller();
			XMLReader rdr = XMLReaderFactory
					.createXMLReader("org.apache.xerces.parsers.SAXParser");
			rdr.setContentHandler(saxUms);
			rdr.parse(src);
			createResult(saxUms.getProject(), saxUms.getTaskList());
		} catch (Exception exc) {
			throw new EMFUserError(EMFErrorSeverity.ERROR,
					"Impossible to parse the document: " + exc.getMessage());
		}
	}

	private void createResult(Project project, List<Task> taskList) {
		for (Task task : taskList) {
			GenericItem genericItem = new GenericItem();
			for (int i = 0; i < Project.elements.length; i++) {
				if (project != null
						&& project.getValue(Project.elements[i]) != null) {
					genericItem.setValue(PROJECT_ + Project.elements[i],
							project.getValue(Task.elements[i]));
				}
			}
			for (int i = 0; i < Task.elements.length; i++) {
				if (task.getValue(Task.elements[i]) != null) {
					genericItem.setValue(TASK_ + Task.elements[i], task
							.getValue(Task.elements[i]));
				}
			}
			result.add(genericItem);
		}
	}

	private String getDocument() throws EMFUserError {
		if (document == null) {
			throw new EMFUserError(EMFErrorSeverity.ERROR,
					"Impossible to create the document.");
		}
		return document;
	}

	@Override
	protected void setUp() {
		setDocument(readOperationParameterValue(FILE));
	}

	protected void setDocument(final String doc) {
		document = doc;
	}

	@Override
	protected void tearDown() {
	}

	@Override
	protected List<GenericItemInterface> extract() throws EMFUserError {
		generateResult();
		return result;
	}
}
